#!/usr/bin/env python
import gtk, gtk.glade
from os.path import *
import gnomeapplet
import getopt, sys
import gconf

# Allow to use uninstalled
def _check(path):
	return exists(path) and isdir(path) and isfile(path+"/AUTHORS")

name = join(dirname(__file__), '..')
if _check(name):
	print 'Running uninstalled autologin, modifying PYTHONPATH'
	sys.path.insert(0, abspath(name))
else:
	sys.path.insert(0, abspath("@PYTHONDIR@"))
	print "Running installed autologin, using [@PYTHONDIR@:$PYTHONPATH]"

# Now the path is set, import our applet


from autologin import connect,has_to_connect
import autologindefs

# Global vars
name = autologindefs.PACKAGE
image_dir = autologindefs.IMAGE_DIR
glade_dir = autologindefs.GLADE_BASE + name
GCONF_DIR = "/apps/autologin"
GCONF_LOGIN = "/login"
GCONF_PASSWORD = "/password"
GCONF_TYPE = "/type"

# Factory applet
def autoLogin_applet_factory(applet, iid):
    print 'Starting AutoLogin instance:', applet, iid
    AutoLoginApplet(applet)
    return True

# Application main class
class AutoLoginApplet:
    def button_press(self,widget,event):
        if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
            self.create_menu()
        elif event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
            if connect(self):
                self.set_button_image("yes")
            else:
                self.set_button_image("no")

    def check_connected(self,event,data=None):
        if has_to_connect(self):
            self.set_button_image("no")
        else:
            self.set_button_image("yes")
    
    # callback for contextual menu
    def create_menu(self):
        self.applet.setup_menu(self.propxml,self.verbs,None)

    # callback of contextual menu for properties dialog
    def show_properties(self,event,data=None):
        self.properties.show_all()

    def set_button_image (self, status):
        # We use an intermediate pixbuf to scale the image
        size = self.applet.get_size()
        filename = self.image_dir + status + ".xpm"
        if self.popup_dir in [gnomeapplet.ORIENT_DOWN, gnomeapplet.ORIENT_UP]:
            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size (filename, -1, size)
        else:
            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size (filename, size, -1)
        self.image.set_from_pixbuf (pixbuf)

    def on_login_changed(self,editable):
        self.login = editable.get_text()
        self.GCONF_CLIENT.set_string(self.GCONF_LOGIN, self.login)
        return gtk.TRUE

    def on_password_changed(self,editable):
        self.password = editable.get_text()
        self.GCONF_CLIENT.set_string(self.GCONF_PASSWORD, self.password)
        return gtk.TRUE

    def on_type_changed(self,editable):
        self.type = editable.get_active()
        self.GCONF_CLIENT.set_int(self.GCONF_TYPE, self.type)
        return gtk.TRUE

    def __init__(self, applet):
        # Contextual menu parameter
        self.propxml="""
            <popup name="button3">
            <menuitem name="Item 1" verb="Props" label="_Properties..." pixtype="stock" pixname="gtk-properties"/>
            <menuitem name="Item 2" verb="Check" label="_Check authentification"/>
            </popup>
            """
        self.verbs = [ ( "Props", self.show_properties ), ("Check", self.check_connected) ]
        
        # Preload gconf parameters
        self.GCONF_CLIENT = gconf.client_get_default ()
        path = applet.get_preferences_key()
        if path != None:
            self.GCONF_APPLET_DIR = path
            print 'Using per-applet gconf key:', self.GCONF_APPLET_DIR
        else:
            self.GCONF_APPLET_DIR = GCONF_DIR
        self.GCONF_LOGIN = self.GCONF_APPLET_DIR + GCONF_LOGIN
        self.GCONF_PASSWORD = self.GCONF_APPLET_DIR + GCONF_PASSWORD
        self.GCONF_TYPE = self.GCONF_APPLET_DIR + GCONF_TYPE
        self.login = self.GCONF_CLIENT.get_string(self.GCONF_LOGIN)
        self.password = self.GCONF_CLIENT.get_string(self.GCONF_PASSWORD)
        self.type = self.GCONF_CLIENT.get_int(self.GCONF_TYPE)
        
        # Properties dialog creation
        wT = gtk.glade.XML(join(glade_dir, "autologin.glade"))
        self.properties = wT.get_widget('properties')
        self.login_editable = wT.get_widget('login')
        self.password_editable = wT.get_widget('password')
        self.type_combo = wT.get_widget('type')
        self.close_bt = wT.get_widget('closebutton')
        self.properties.connect("delete_event",self.properties.hide_on_delete)
        if self.login != None:
            self.login_editable.set_text(self.login)
        self.login_editable.connect("changed",self.on_login_changed)
        if self.password != None:
            self.password_editable.set_text(self.password)
        self.password_editable.connect("changed",self.on_password_changed)
        if self.type != None:
            self.type_combo.set_active(self.type)
        self.type_combo.value_in_list = True
        self.type_combo.connect("changed",self.on_type_changed)
        self.close_bt.connect("clicked",self.properties.hide_on_delete)

        # Widgets creation
        self.applet = applet
        # One box for panel orientation
        self.popup_dir = applet.get_orient()
        if self.popup_dir in [gnomeapplet.ORIENT_UP,gnomeapplet.ORIENT_DOWN]:
            self.box = gtk.HBox ()
        else:
            self.box = gtk.VBox ()
        self.applet.add(self.box)

        # Event Box
        self.evBox = gtk.EventBox()
        self.evBox.connect("button-press-event",self.button_press)
        self.box.add(self.evBox)
        
        # Image
        self.image_dir = image_dir
        self.image = gtk.Image()
        self.check_connected(None,None)
        self.evBox.add(self.image)

        # Show it all
        self.applet.show_all()

# Return a standalone window that holds the applet
def build_window():
    app = gtk.Window(gtk.WINDOW_TOPLEVEL)
    app.set_title("AutoLogin Applet")
    app.connect("destroy", gtk.main_quit)
    applet = gnomeapplet.Applet()
    applet.get_orient = lambda: gnomeapplet.ORIENT_DOWN
    autoLogin_applet_factory(applet, None)
    applet.reparent(app)   
    app.show_all()
    return app
    
def usage():
    print """=== AutoLogin applet: Usage
$ autologin-applet [OPTIONS]

OPTIONS:
    -h, --help        Print this help notice.
    -w, --window      Launch the applet in a standalone window for test purposes (default=no).
    """
    sys.exit()

if __name__ == "__main__":    
    standalone = False
    
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hw", ["help", "window"])
    except getopt.GetoptError:
        # Unknown args were passed, we fallback to bahave as if
        # no options were passed
        print "WARNING: Unknown arguments passed, using defaults."
        opts = []
        args = sys.argv[1:]
    
    for o, a in opts:
        if o in ("-h", "--help"):
            usage()
        elif o in ("-w", "--window"):
            standalone = True

    print 'Running with options:', {
        'standalone': standalone
        }
    
    if standalone:
        import gnome
        gnome.init(autologindefs.PACKAGE, autologindefs.VERSION)
        build_window()
        gtk.main()     
    else:
        gnomeapplet.bonobo_factory(
               "OAFIID:GNOME_AutoLogin_Factory",
               gnomeapplet.Applet.__gtype__,
               autologindefs.PACKAGE,
               autologindefs.VERSION,
               autoLogin_applet_factory)
